package core;
/*
 * This file is part of CANE - A Clustering Algorithm for Non-Euclidean spaces.
 *
 * Copyright (C) 2014 Benjamin Eltzner
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 * or see <http://www.gnu.org/licenses/>.
 */


/**
 * This exception is accessible by two class methods.
 * 
 * * throw_new should be used be Clusterables and SimpleClusterables
 *   that do not implement the optional methods that can throw this
 *   exception.
 * * called_by should be used in try-catch-blocks of CANE functions
 *   to document both points of failure for the exception.
 * 
 * This two step procedure is supposed to produce an expressive
 * error message.
 */
public class NotImplementedException extends RuntimeException
{

    private static final long serialVersionUID = -42L;

    private NotImplementedException(String message)
    {
        super(message);
    }

    /**
     * Factory method for NotImplementedException. Should only be
     * called from Clusterables and SimpleClusterables to produce
     * the initial error message.
     * @param unimplemented_method  The name of the method of the
     * Clusterable or SimpleClusterable which is unimplemented.
     * @return NotImplementedException
     */
    public static NotImplementedException throw_new(String unimplemented_method)
    {
        return new NotImplementedException("method " + unimplemented_method +
                                           " is not implemented.");
    }

    /**
     * Factory method for NotImplementedException. Should only be
     * called from try-catch blocks. Eats an incoming
     * NotImplementedException and returns a new one.
     * @param e         A NotImplementedException coming from a
     * method call of a Clusterable or SimpleClusterable.
     * @param action    String that specifies the operation which
     * is unsupported due to the unimplemented method in the
     * Clusterable or SimpleClusterable.
     * @return NotImplementedException
     */
    public static NotImplementedException called_by(NotImplementedException e,
                                                    String action)
    {
        return new NotImplementedException(action + " is not possible because " +
                                           e.getMessage());
    }
}
