package core;
/*
 * This file is part of FilamentSensor - A tool for filament tracking from cell images
 *
 * Copyright (C) 2016 Benjamin Eltzner
 *
 * FilamentSensor is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 * or see <http://www.gnu.org/licenses/>.
 */


import java.awt.Point;
import java.util.List;

public abstract class SingleFilament extends AbstractFilament
{
    public abstract ClusterableFilament makeClusterable(int time);
    public abstract List<Point> getPoints();
    public abstract double getMass();
    public long getSignedCurvature() { return 0; }
    public long getAbsoluteCurvature() { return 0; }
    public void drawToImage (boolean[][] image)
    {
        List<Point> points = getPoints();
        for (int i = 0; i < points.size() - 1; i++)
        {
            Point p_1 = points.get(i), p_2 = points.get(i+1);
            int d_x = p_2.x - p_1.x, d_y = p_2.y - p_1.y, d = Math.max(Math.abs(d_x),  Math.abs(d_y));
            double step_x = d_x / (double)d, step_y = d_y / (double)d, p_x = p_1.x, p_y = p_1.y;
            for (int k = 0; k <= d; k++)
            {
                image[(int)Math.round(p_x)][(int)Math.round(p_y)] = true;
                p_x += step_x;
                p_y += step_y;
            }
        }
    }
    public abstract List<Filament> splitIntoLinearPieces();
}

    