package filters;
/*
 * This file is part of FilamentSensor - A tool for filament tracking from cell images
 *
 * Copyright (C) 2016 Benjamin Eltzner
 *
 * FilamentSensor is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 * or see <http://www.gnu.org/licenses/>.
 */


import java.awt.Point;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class FilterCloseHoles
{

    public FilterCloseHoles()
    {
    }

    public static final boolean[][] apply(boolean[][] binary_image)
    {
        if (binary_image == null || binary_image.length == 0 || binary_image[0].length == 0)
        {
            System.err.println("FilamentSensor FilterCloseHoles.apply: " +
                               "Image must exist and have non-zero size.");
            return null;
        }

        final int width = binary_image.length,
                  height = binary_image[0].length;

        boolean[][] out = new boolean[width][height];
        int count = 0;
        for (int x = 0; x < width; x++)
        {
            for (int y = 0; y < height; y++)
            {
                if (binary_image[x][y])
                {
                    out[x][y] = true;
                    count++;
                }
            }
        }

        int[][] map = new int[width][height];
        Map<Integer,List<Point>> fragments = new HashMap<>();
        int index = 0;
        for (int x = 0; x < width; x++)
        {
            for (int y = 0; y < height; y++)
            {
                if (!binary_image[x][y])
                {
                    if (x > 0 && !binary_image[x-1][y])
                    {
                        int label = map[x][y] = map[x-1][y];
                        fragments.get(map[x][y]).add(new Point(x,y));
                        if (y > 0 && label != map[x][y-1] && fragments.containsKey(map[x][y-1]))
                        {
                            List<Point> to_dissolve = fragments.remove(map[x][y-1]);
                            for (Point p : to_dissolve)
                            {
                                map[p.x][p.y] = label;
                            }
                            fragments.get(map[x][y]).addAll(to_dissolve);
                        }
                    }
                    else if (y > 0 && !binary_image[x][y-1])
                    {
                        map[x][y] = map[x][y-1];
                        fragments.get(map[x][y]).add(new Point(x,y));
                    }
                    else
                    {
                        fragments.put(++index, new ArrayList<Point>());
                        map[x][y] = index;
                        fragments.get(map[x][y]).add(new Point(x,y));
                    }
                }
            }
        }

        for (List<Point> fragment : fragments.values())
        {
            if (fragment.size() < count)
            {
                for (Point p : fragment)
                {
                    out[p.x][p.y] = true;
                }
            }
        }
        return out;
    }
}
