package installer;
/*
 * LoadJarFile- installer script locate the ImageJ library 
 *
 * Copyright (C) 2014  Nguyen Chuong Thach
 *
 * FilTracker is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 * or see <http://www.gnu.org/licenses/>.
 */


import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.attribute.PosixFilePermission;
import java.util.HashSet;
import java.util.Set;

public class LoadJarFile
{

    private static final String DIR = System.getProperty("user.dir");
    private static final String OS = System.getProperty("os.name").toLowerCase();
    private static final String sep = File.separator;
    private static final String path = File.pathSeparator;
    private static final String ls = System.lineSeparator();

    public LoadJarFile() 
    {
        JFrame frame = new JFrame("Filament Sensor Installer");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setVisible(true);

        String ij_jar = openDialog(frame, "Locate ImageJ (ij.jar)");
        if (ij_jar == null) { return; }
        String fs_jar = openDialog(frame, "Locate Filament Sensor (FilamentSensor.jar)");
        if (fs_jar == null) { return; }
        String call = "java -cp \"" + fs_jar + path + ij_jar + "\" ui.FilamentSensor";
        if (OS.indexOf("win") >= 0) // Check Window OS
        {
            try 
            {
                writeScriptFileWin("@ECHO OFF" + ls +
                                   "setLocal EnableDelayedExpansion" + ls +
                                   call + " %*" + ls);
            } 
            catch (IOException ex) 
            {
                ex.printStackTrace();
            }
        }
        else // Unixoid 
        {
            try 
            {
                writeScriptFileUnix("#!/bin/sh"+ ls +
                                    call + " $@" + ls);
            }
            catch (IOException ex) 
            {
                ex.printStackTrace();
            }
        }
    }

    private static String openDialog(JFrame frame, String caption)
    {
        JFileChooser fc = new JFileChooser();
        fc.setDialogTitle(caption);
        fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fc.setAcceptAllFileFilterUsed(false);
        fc.setCurrentDirectory(new File(DIR));

        if (fc.showOpenDialog(frame) == JFileChooser.APPROVE_OPTION)
        {
            if (fc.getSelectedFile() == null)
            {
                return null;
            }
            return fc.getSelectedFile().getAbsolutePath();
        }
        return null;
    }
    
    private void writeScriptFileWin(String content) throws IOException
    {
        File file = new File(DIR + sep + "FilamentSensor.bat");
        BufferedWriter bwr = new BufferedWriter(new FileWriter(file, false));
        bwr.write(content);
        bwr.close();
    }
    
    private void writeScriptFileUnix(String content) throws IOException
    {
        File file = new File(DIR + sep + "FilamentSensor.sh");
        BufferedWriter bwr = new BufferedWriter(new FileWriter(file, false));
        bwr.write(content);
        bwr.close();

        // set Permission
        Set<PosixFilePermission> perms = new HashSet<PosixFilePermission>();
        perms.add(PosixFilePermission.OWNER_READ);
        perms.add(PosixFilePermission.OWNER_WRITE);
        perms.add(PosixFilePermission.OWNER_EXECUTE);
        Files.setPosixFilePermissions(Paths.get(file.getAbsolutePath()), perms);
    }
 
    public static void main(String[] args) 
    {
        //Schedule a job for the event dispatch thread:
        //creating and showing this application's GUI.
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                //Turn off metal's use of bold fonts
                UIManager.put("swing.boldMetal", Boolean.FALSE);
                new LoadJarFile();
                System.exit(0);
            }
        });
    }
}
