package ui;
/*
 * This file is part of FilamentSensor - A tool for filament tracking from cell images
 *
 * Copyright (C) 2011-2013 Julian Rüger
 *               2013      Patricia Burger, Benjamin Eltzner
 *
 * FilamentSensor is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 * or see <http://www.gnu.org/licenses/>.
 */

import java.awt.Component;
import java.awt.Dimension;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.plaf.basic.BasicButtonUI;

abstract class AbstractTab extends JPanel
{
    protected String m_tab_title;
    protected JLabel m_tab_label;
    protected TabBar m_tab_bar;

    protected AbstractTab(LayoutManager flow_layout)
    {
        super(flow_layout);
    }

    boolean readyToClose(boolean quiet)
    {
        return true;
    }

    protected void setTitle(String title)
    {
        m_tab_title = m_tab_bar.setTitle(this, title);
        // m_tab_label.setText((m_data.hasChanged() ? "*" : "")+title);
    }

    String getTitle()
    {
        return m_tab_title;
    }

    void setTooltip(String tooltip)
    {
        m_tab_bar.setTooltip(this, tooltip);
    }

    // Unclear where exactly it is used
    protected final static MouseListener buttonMouseListener = new MouseAdapter()
    {
        // Called just after the cursor enters the bounds of the component
        @Override
        public void mouseEntered(MouseEvent e)
        {
            Component component = e.getComponent();
            if (component instanceof AbstractButton)
            {
                AbstractButton button = (AbstractButton) component;
                // Highlights button if cursor enters its bounds
                button.setBorderPainted(true);
                button.setContentAreaFilled(true);
            }
        }

        // Called just after the cursor exits the bounds of the component
        @Override
        public void mouseExited(MouseEvent e)
        {
            Component component = e.getComponent();
            if (component instanceof AbstractButton)
            {
                AbstractButton button = (AbstractButton) component;
                // Makes button look normal again if cursor exits its bounds
                button.setBorderPainted(false);
                button.setContentAreaFilled(false);
            }
        }
    };

    /**
     * Private class that handles the close-buttons on tabs, their appearance
     * and ActionEvents.
     */
    protected class TabButton extends JButton implements ActionListener
    {
        public TabButton()
        {
            super(IconCollection.CLOSE16);
            setPreferredSize(new Dimension(16, 16));
            setToolTipText("Close this tab");

            this.setUI(new BasicButtonUI());
            setContentAreaFilled(false);

            setFocusable(false);
            setBorder(BorderFactory.createEtchedBorder());
            setBorderPainted(false);

            // Add MouseListener for the "rollover"-effect
            addMouseListener(buttonMouseListener);
            setRolloverEnabled(true);

            addActionListener(this);
        }

        @Override
        public void actionPerformed(ActionEvent e)
        {
            m_tab_bar.closeTab(m_tab_bar.indexOfTabComponent(AbstractTab.this), false);
        }
    }

}
