package ui;
/*
 * This file is part of FilamentSensor - A tool for filament tracking from cell images
 *
 * Copyright (C) 2013-2015 Benjamin Eltzner
 *
 * FilamentSensor is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 * or see <http://www.gnu.org/licenses/>.
 */


import java.awt.Dimension;
import java.awt.Point;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.awt.image.BufferedImage;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

public class PopUpDebug extends JFrame implements ActionListener
{
    private JPanel m_panel_content, m_panel_controls;
    private PanelImage m_panel_image;
    private JButton m_button_save;

    public PopUpDebug(boolean[][] binary_image, boolean invert, String name)
    {
        super(name);
        setUp();
        m_panel_image.setImage(ImageFactory.makeBinaryImage(binary_image, invert), true);
    }

    public PopUpDebug(int[][] int_image, String name)
    {
        super(name);
        setUp();
        m_panel_image.setImage(ImageFactory.makeColorImage(int_image, false), true);
    }

    public PopUpDebug(BufferedImage image, String name)
    {
        super(name);
        setUp();
        m_panel_image.setImage(image, true);
    }

    private void setUp()
    {
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);

        m_panel_content = new JPanel();
        m_panel_content.setLayout(new BoxLayout(m_panel_content, BoxLayout.Y_AXIS));

        m_panel_image = new PanelImage();
        m_panel_content.add(m_panel_image);

        m_panel_controls = new JPanel();
        m_panel_controls.setLayout(new BoxLayout(m_panel_controls, BoxLayout.X_AXIS));

        m_panel_controls.add(Box.createHorizontalGlue());
        m_button_save = new JButton("  Save  ");
        m_button_save.addActionListener(this);
        m_panel_controls.add(m_button_save);

        m_panel_content.add(m_panel_controls);

        setDefaultLookAndFeelDecorated(false);
        setContentPane(m_panel_content);

        setMinimumSize(new Dimension(MainWindow.MIN_WIDTH, MainWindow.MIN_HEIGHT));
        Dimension size = MainWindow.frame().getSize();
        size.width -= 100;
        size.height -= 100;
        this.setSize(size);
        Point location = MainWindow.frame().getLocation();
        location.x += 50;
        location.y += 50;
        this.setLocation(location);
        setVisible(true);
    }

    @Override
    public void actionPerformed(ActionEvent e)
    {
        Object src = e.getSource();

        if (src == m_button_save)
        {
            Dialogs.saveImage(m_panel_image.getImage());
        }
    }
}