package ui;
/*
 * This file is part of FilamentSensor - A tool for filament tracking from cell images
 *
 * Copyright (C) 2013 Benjamin Eltzner
 *
 * FilamentSensor is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 * or see <http://www.gnu.org/licenses/>.
 */


import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

public class PopUpOrientationFields extends JFrame implements ActionListener
{
    private JPanel m_panel_content, m_panel_controls;
    private PanelImage m_panel_image;
    private JButton m_next_image, m_previous_image;
    private JLabel m_display_image_number;
    private List<BufferedImage> m_images;
    private int m_current_image;

    public PopUpOrientationFields(List<BufferedImage> images)
    {
        super("Orientation Fields");
        setUp();
        m_images = images;
        if (m_images != null && !m_images.isEmpty())
        {
            m_current_image = 0;
            m_display_image_number.setText("Image " + (m_current_image + 1) +
                                           " / " + m_images.size());
            m_panel_image.setImage(m_images.get(m_current_image), true);
        }
    }

    private void setUp()
    {
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);

        m_panel_content = new JPanel();
        m_panel_content.setLayout(new BoxLayout(m_panel_content, BoxLayout.Y_AXIS));

        m_panel_image = new PanelImage();
        m_panel_content.add(m_panel_image);

        m_panel_controls = new JPanel();
        m_panel_controls.setLayout(new BoxLayout(m_panel_controls, BoxLayout.X_AXIS));

        m_panel_controls.add(Box.createHorizontalGlue());
        m_previous_image = new JButton("  Previous  ");
        m_previous_image.addActionListener(this);
        m_panel_controls.add(m_previous_image);
        m_panel_controls.add(Box.createRigidArea(new Dimension(0, 10)));
        m_next_image = new JButton("  Next  ");
        m_next_image.addActionListener(this);
        m_panel_controls.add(m_next_image);
        m_panel_controls.add(Box.createRigidArea(new Dimension(0, 10)));
        m_display_image_number = new JLabel();
        m_panel_controls.add(m_display_image_number);
        m_panel_controls.add(Box.createHorizontalGlue());

        m_panel_content.add(m_panel_controls);

        setDefaultLookAndFeelDecorated(false);
        setContentPane(m_panel_content);

        setMinimumSize(new Dimension(MainWindow.MIN_WIDTH, MainWindow.MIN_HEIGHT));
        Dimension size = MainWindow.frame().getSize();
        size.width -= 100;
        size.height -= 100;
        this.setSize(size);
        Point location = MainWindow.frame().getLocation();
        location.x += 50;
        location.y += 50;
        this.setLocation(location);
        setVisible(true);
    }

    @Override
    public void actionPerformed(ActionEvent e)
    {
        Object src = e.getSource();

        if (src.equals(m_previous_image))
        {
            if (m_current_image == 0)
            {
                m_current_image = m_images.size() - 1;
            }
            else
            {
                m_current_image--;
            }
            m_panel_image.setImage(m_images.get(m_current_image), true);
            m_display_image_number.setText("Image " + (m_current_image + 1) +
                                           " / " + m_images.size());
        }
        else if (src.equals(m_next_image))
        {
            if (m_current_image == m_images.size() - 1)
            {
                m_current_image = 0;
            }
            else
            {
                m_current_image++;
            }
            m_panel_image.setImage(m_images.get(m_current_image), true);
            m_display_image_number.setText("Image " + (m_current_image + 1) +
                                           " / " + m_images.size());
        }
    }
}