package ui;
/*
 * A combination of JSlider and JSpinner (in a JPanel).
 *
 * Copyright 2012 Julian Rüger
 *
 * This class is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This class is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this class; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 * or see <http://www.gnu.org/licenses/>.
 */


import java.awt.Dimension;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


public class SlideSpinPane extends JPanel implements ChangeListener
{
    private JSlider slider;
    private JSpinner spinner;
    private String event_command;
    private ActionListener listener;

    private boolean changing = false;
    private int min, max;

    public SlideSpinPane(String title, int initial, int min, int max, int step, int soft_min, int soft_max)
    {
        super();
        this.min = min;
        this.max = max;
        listener = null;
        setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
        setBorder(BorderFactory.createTitledBorder(title));

        slider = new JSlider(SwingConstants.HORIZONTAL, soft_min, soft_max, initial);
        slider.setPaintTicks(true);
        slider.setPaintLabels(true);
        slider.addChangeListener(this);

        spinner = new JSpinner(new SpinnerNumberModel(initial, min, max, step));
        spinner.setMaximumSize(new Dimension(50, 28));
        // FIXME: adaptive width depending on max^(.1)
        spinner.addChangeListener(this);

        this.add(slider);
        this.add(Box.createHorizontalStrut(5));
        this.add(spinner);
    }

    public SlideSpinPane(String title, int initial, int min, int max, int step)
    {
        this(title, initial, min, max, step, min, max);
    }

    public void setTickSpacing(int minor, int major)
    {
        slider.setMinorTickSpacing(minor);
        slider.setMajorTickSpacing(major);
    }

    public void addActionListener(ActionListener l, String command)
    {
        listener = l;
        event_command = command;
    }

    @Override
    public void setEnabled(boolean value)
    {
        slider.setEnabled(value);
        spinner.setEnabled(value);
    }

    /**
     *
     */
    @Override
    public void stateChanged(ChangeEvent e)
    {
        if (changing)
        {
            return;
        }

        Object src = e.getSource();

        if (src == slider)
        {
            changing = true;
            spinner.setValue(Math.min(max, Math.max(min, slider.getValue())));
            slider.setValue(Math.min(max, Math.max(min, slider.getValue())));
            changing = false;
        }
        else
        {
            changing = true;
            slider.setValue((Integer) spinner.getValue());
            changing = false;
        }

        if (listener != null && !slider.getValueIsAdjusting())
        {
            listener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, event_command));
        }
    }

    public int getValue()
    {
        return (Integer) spinner.getValue();
    }

    public void setValue(int value)
    {
        spinner.setValue(value);
    }

}
