package ui;
/*
 * A combination of JSlider and JSpinner (in a JPanel) - Double (instead of Integer) version.
 *
 * Copyright 2012 Julian Rüger
 *
 * This class is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This class is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this class; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 * or see <http://www.gnu.org/licenses/>.
 */


import java.awt.Dimension;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Hashtable;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


public class SlideSpinPaneDouble extends JPanel implements ChangeListener
{
    private JSlider slider;
    private JSpinner spinner;
    private String event_command;
    private ActionListener listener;

    private double step_factor;
    private boolean changing = false;

    private Hashtable<Integer, JLabel> labels;

    public SlideSpinPaneDouble(String title, double initial, double min, double max, double step, double soft_min,
            double soft_max)
    {
        super();
        listener = null;
        setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
        setBorder(BorderFactory.createTitledBorder(title));

        step_factor = Math.min(step, 1.0);

        slider = new JSlider(SwingConstants.HORIZONTAL, this.conv(soft_min), this.conv(soft_max), this.conv(initial));
        slider.setPaintTicks(true);
        slider.setPaintLabels(true);
        slider.addChangeListener(this);

        spinner = new JSpinner(new SpinnerNumberModel(initial, min, max, step));
        spinner.setMaximumSize(new Dimension(50, 28));
        // FIXME: adaptive  width depending on max^(.1)
        spinner.addChangeListener(this);

        this.add(slider);
        this.add(Box.createHorizontalStrut(5));
        this.add(spinner);
    }

    public SlideSpinPaneDouble(String title, double initial, double min, double max, double step)
    {
        this(title, initial, min, max, step, min, max);
    }

    private int conv(double value)
    {
        return (int)Math.round(value / step_factor);
    }

    private int conv(Object value)
    {
        return this.conv(((Number) value).doubleValue());
    }

    public void setTickSpacing(double minor, double major)
    {
        if (labels == null)
        {
            labels = new Hashtable<Integer, JLabel>();
            for (double i = slider.getMinimum(); i <= slider.getMaximum(); i += major)
            {
                labels.put(this.conv(i), new JLabel(String.format("%3.1f", i)));
            }
        }

        slider.setMinorTickSpacing(Math.max(1, this.conv(minor)));
        slider.setMajorTickSpacing(this.conv(major));
        slider.setLabelTable(labels);
    }

    public void addActionListener(ActionListener l, String command)
    {
        listener = l;
        event_command = command;
    }

    @Override
    public void setEnabled(boolean value)
    {
        slider.setEnabled(value);
        spinner.setEnabled(value);
    }

    /**
     *
     */
    @Override
    public void stateChanged(ChangeEvent e)
    {
        if (changing)
        {
            return;
        }

        Object src = e.getSource();

        if (src == slider)
        {
            changing = true;
            spinner.setValue(slider.getValue() * step_factor);
            changing = false;
        }
        else
        {
            changing = true;
            slider.setValue(this.conv(spinner.getValue()));
            changing = false;
        }

        if (listener != null && !slider.getValueIsAdjusting())
        {
            listener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, event_command));
        }
    }

    public double getValue()
    {
        return (Double) spinner.getValue();
    }

    public void setValue(double value)
    {
        spinner.setValue(value);
    }

}
